/* Pxys - An IRC ProxyScan, mp_negativity.c
 * 
 * Copyright (C) 2003 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 *
 *
 * $Id: mp_negativity.c,v 1.2 2003/06/26 17:47:46 mbuna Exp $
 */


#define NEGATIVITY_PORT	407

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifdef HAVE_SYS_POLL_H
#include <sys/poll.h>
#endif

#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>

#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#include "conf.h"
#include "debug.h"
#include "typedefs.h"

#include "mp_poll.h"
#include "mp_negativity.h"
#include "mp_utils.h"
#include "m_log.h"
#include "m_scanner.h"
#include "m_shutdown.h"
#include "ms_globals.h"
#include "ms_msgout.h"

extern int unreach_count;


void
scan_negativity(scanthread_s * sth)
    {
    int i, nfds, err;
    unsigned int length;
    time_t now;
    struct pollfd* pfd;
    char readbuf[128];
    Boolean result = false;
    Boolean timed_out;
   
    scanItems_init(sth, POLLWRITEFLAGS, NEGATIVITY_PORT);
   
    /* And start poll lookup loop */
    for (;;)
        {
        usleep(sth->counter * 2500);
        nfds = poll(sth->poll_fds, sth->counter, POLL_TIMEOUT);
      
        if (nfds <= -1)
            {
            if (errno == EINTR)
                continue;
            log_error("scan_wingate: poll error %d (%s)\n", errno, strerror(errno));
            break;
            }
      
        /* Timeout checking */
        time(&now);
        timed_out = true;
        for (i = 0; i < sth->counter; i++)
            {
            struct scanItem *sci = &sth->item[i];
         
            if (sci->state & (SIT_ERROR|SIT_FOUND))
                continue;
            if (now >= sci->timeout)
                sci->state |= SIT_TIMEOUT;
         
            timed_out = timed_out && (sci->state & SIT_TIMEOUT);
            }
      
        /* All fds are timed out, exiting. */
        if (timed_out)
            break;
      
        pfd = sth->poll_fds;

        for (i = 0; nfds && (i < sth->counter); ++i, ++pfd)
            {
            struct scanItem *sci = &sth->item[i];
         
            if (pfd->revents & POLLERRORS)
                {
                log_error("mp_negativity: POLLERRORS %s", inet_ntoa(sci->addr));
                sci->state |= SIT_ERROR;
                DISABLE_PFD(pfd);
                nfds--;
                continue;
                }
         
            if (sci->state & (SIT_ERROR|SIT_FOUND)) /* skip inactives */
                continue;
         
            if ((err = get_sockerr(pfd->fd)))
                {
                if (err == EHOSTUNREACH)
                    {
                    unreach_count++;
                    sci->state |= SIT_UNREACH;
                    sci->dont_cache = 1;
                    log_write(LOGID_UNREACH, "%s", inet_ntoa(sci->addr));
                    }
                else if (err != ECONNREFUSED)
                    {
                    log_error("mp_negativity: get_sockerr %s [%d] (%s)",
                              inet_ntoa(sci->addr), err, strerror(err));
                    }
                sci->state |= SIT_ERROR;
                DISABLE_PFD(pfd);
                nfds--;
                continue;
                }
         
            if ((pfd->events & POLLWRITEFLAGS) && (pfd->revents & POLLWRITEFLAGS))
                {
                if (IsConnecting(sci)) {    /* Are we really connected? */
                    SetConnected(sci);
                    pfd->events = POLLREADFLAGS; /* We want to read */
                    pfd->revents = 0;
                    continue;
                    }
                }
         
            if ((pfd->events & POLLREADFLAGS) && (pfd->revents & POLLREADFLAGS))
                {
            
                /* Read ready: receive data */
                memset(readbuf, 0, sizeof(readbuf));
                err = recv_nonb(pfd->fd, readbuf, sizeof(readbuf) - 1, &length);
                if (err > 0)
                    {
                    sci->tryread++;
                    if (length > 0)
                        {
/* :62.179.153.226 NOTICE AUTH :Anda perlu taip /quote PASS <password>
 */
                        char *p = readbuf;
                        int c;
                        
                        while ((c = *p++) && c != ' ')
                            ;
                        
                        if (c && (!strncmp(p, "NOTICE AUTH :Anda perlu taip", 28)
                                  || !strncmp(p, "NOTICE AUTH :Sila taip", 22)))
                            {
                            sci->state |= SIT_FOUND;
                            proxy_toast(&sci->addr, PROXY_TYPE_NEGATIVITY, 0);
                            Debug((DBLMAX, "scan_negativity:recv_nonb Negativity bot found (%d): %s",
                                   sci->tryread, readbuf));
                            result = true;
                            sci->tryread = 2;
                            }
                        else
                            {
                            log_error("mp_negativity: readbuf='%s'", readbuf);
                            }
                        if (sci->tryread >= 2)
                            {
                            shutdown(pfd->fd, SHUT_RDWR);
                            DISABLE_PFD(pfd);
                            nfds--;
                            continue;
                            }
                        }
                    }
                else if (err == 0)
                    {
                    /* We want to retry. */
                    pfd->events = POLLREADFLAGS;
                    pfd->revents = 0;
                    }
                else
                    {
                    DISABLE_PFD(pfd);
                    nfds--;
                    }
                }
            }
        }
    scanItems_closeall(sth);
    }

/* mp_negativity.c */
