/* Pxys, An IRC ProxyScan, mp_utils.h
 * 
 * Copyright (C) 2001,2002 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * $Id: mp_utils.c,v 1.24 2002/12/16 01:10:17 mbuna Exp $
 */
 
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>   /* strerror */
#include <unistd.h>   /* close */
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#ifdef HAVE_FCNTL_H
#   include <fcntl.h>
#endif
#ifdef HAVE_PTHREAD_H
#   include <pthread.h>
#endif
#ifdef HAVE_SYS_POLL_H
#include <sys/poll.h>
#endif
#ifdef HAVE_SYS_FILIO_H
#include <sys/filio.h> /* FIONBIO on solaris */
#endif

#include <sys/ioctl.h>
#include <errno.h>

#include "typedefs.h"
#include "conf.h"
#include "debug.h"

#include "mp_utils.h"

#include "m_structs.h"
#include "m_log.h"
#include "m_shutdown.h"

#include "ms_msgout.h"

/* Errors counters for statistical/debugging purpose */
int recv_eintr_count = 0, /* recv got EINTR */
    send_eintr_count = 0, /* send got EINTR */
    recv_epipe_count = 0, /* recv got EPIPE */
    send_epipe_count = 0, /* send got EPIPE */
    recv_err_count = 0,   /* recv got EWOULDBLOCK or EAGAIN */
    send_err_count = 0;   /* send got EWOULDBLOCK, EAGAIN, ENOMEM or ENOBUFS */

void
scanItems_init(
   scanthread_s * sth,
   int events,
   unsigned short port)
{
   int fd, i, optval, cnt;
   struct sockaddr_in localAddr;
   
   if (Me->vhost) {
      memset(&localAddr, 0, sizeof(struct sockaddr_in));
      localAddr.sin_family = AF_INET;
      localAddr.sin_addr.s_addr = Me->bindaddr;
      localAddr.sin_port = htons(0);
   }
   
   for (i = 0; i < sth->counter; i++) {
      struct scanItem *sci = &sth->item[i];
      
      if (sci->state & SIT_UNREACH)
         sci->state |= SIT_INACTIVE;
      
      if (sci->state & (SIT_INACTIVE|SIT_FOUND|SIT_NOSOCKS)) {
         sth->poll_fds[i].fd = -1;
         continue;
      }
      
      /* Create socket */
      errno = 0;
      fd = socket(AF_INET, SOCK_STREAM, 0);
      if (fd < 0) {
         optval = errno;
         sendto_console("[%s] socket() failed (%s)", Me->id, strerror(optval));
         scanItems_closeall(sth);
         log_error("mp_utils:scanItems_init: cannot open socket, aborting. (errno = %d - %s)",
              optval, strerror(optval));
         m_shutdown();
         return;
      }
      
      sth->poll_fds[i].fd = fd;
      sth->poll_fds[i].events = events;
      sth->poll_fds[i].revents = 0;
      
      /* Are we using a vhost? if so, bind localAddr any port number */
      if (Me->vhost) {
         int result = -1;
         errno = 0;
         cnt = 10;
         for (cnt = 10; result < 0 && cnt > 0; cnt--) {
             /* bind do not modify localAddr (const) */
	         result = bind(fd, (struct sockaddr *) &localAddr, sizeof(localAddr));
	         usleep(8000);
	     }
         if (result < 0) {
            log_error("mp_utils:scanItems_init: bind failed for %s (errno=%d (%s))",
                 inet_ntoa(localAddr.sin_addr), errno, strerror(errno));
            m_shutdown();
            return;
         }
      }
      
      memset(&sci->clientAddr, 0, sizeof(struct sockaddr_in));
      sci->clientAddr.sin_family = AF_INET;
      sci->clientAddr.sin_addr.s_addr = sci->addr.s_addr;
      sci->clientAddr.sin_port = htons(port);
      time(&sci->timeout); /* Reentrant */
      sci->timeout += MyPerform->timeout_connect;
      sci->tryread = 0;
      sci->retry = 10; /* Retry 10 times if a socket operation fails with EINTR or EPIPE */
      sci->dont_cache = 0;
      
      if (set_nonblock_flag(fd) == -1) {
         sci->state |= SIT_ERROR;
         log_error("mp_utils:scanItems_init:set_nonblock_flag failed (errno %d)", errno);
         Debug((DBLMAX, "mp_utils:scanItems_init:set_nonblock_flag failed (errno %d)", errno));
         continue;
      }

#if defined( SOL_SOCKET ) && defined( SO_REUSEADDR )
      optval = 1;
      (void)setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
#endif
      
      if (!connect_nonb(fd, &sci->clientAddr)) {
         sci->state |= SIT_ERROR;
         log_error("mp_utils:scanItems_init: connect_nonb failed (errno=%d)", errno);
         continue;
      }
   }
}

void
scanItems_closeall(scanthread_s * sth)
{
   int i;
   
   for (i = 0; i < sth->counter; ++i) {
      if (sth->poll_fds[i].fd != -1) {
         /* shutdown */
         /* (void) shutdown(sth->poll_fds[i].fd, 2); */
      
         /* close fd */
         myclose(sth->poll_fds[i].fd);
      }
      
      /* reset some volatile flags :) */
      if (sth->item[i].state & SIT_TIMEOUT)
         sth->item[i].state &= ~SIT_TIMEOUT;
      if (sth->item[i].state & SIT_ERROR)
         sth->item[i].state &= ~SIT_ERROR;
      if (sth->item[i].state & SIT_INPROGRESS)
         sth->item[i].state &= ~SIT_INPROGRESS;
      if (sth->item[i].state & SIT_CONNECTED)
         sth->item[i].state &= ~SIT_CONNECTED;
   }
}


int
set_nonblock_flag(int desc)
{
   int res;
#ifndef NBLOCK_SYSV
   int nonb = 0;
#endif

#ifdef  NBLOCK_POSIX
   nonb |= O_NONBLOCK;
#endif
#ifdef  NBLOCK_BSD
   nonb |= O_NDELAY;
#endif
#ifdef  NBLOCK_SYSV
   res = 1;

   if (ioctl(desc, FIONBIO, &res) == -1)
      return 0;
#else
   if ((res = fcntl(desc, F_GETFL, 0)) == -1)
      return 0;
   else if (fcntl(desc, F_SETFL, res | nonb) == -1)
      return 0;
#endif
   return 1;
}

int
get_sockerr(int fd)
{
  int    err = 0;
#if defined(SO_ERROR)
  unsigned int len = sizeof(err);
  getsockopt(fd, SOL_SOCKET, SO_ERROR, &err, &len);
#endif
  return err;
}

int
connect_nonb(int fd, const struct sockaddr_in* sin)
{
  errno = 0;
  if (connect(fd, (struct sockaddr*) sin, sizeof(struct sockaddr_in))) {
     if (errno != EINPROGRESS && errno != 0) /* Solaris workaround.. */
        return 0;
  }
  return 1;
}

int
set_sockbufs(int fd, unsigned int size)
{
  unsigned int opt = size;
  return (0 == setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (const char*) &opt, sizeof(opt)) &&
          0 == setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (const char*) &opt, sizeof(opt)));
}

int
recv_nonb(int fd, char* buf, unsigned int length, unsigned int* count_out)
{
  int res;
  *count_out = 0;
  errno = 0;

  if (0 < (res = recv(fd, buf, length, 0))) {
    *count_out = (unsigned int) res;
    return 1;
  } else if (res < 0) {
    if (EWOULDBLOCK == errno || EAGAIN == errno) {
       send_err_count++;
       return 0;
    } else if (EINTR == errno) {
       recv_eintr_count++;
       return 0;
    } else if (EPIPE == errno) {
       recv_epipe_count++;
       return 0;
    }
  } 
  return -1;
}

int
send_nonb(int fd, const char* buf, unsigned int length, unsigned int* count_out)
{
  int res;
  *count_out = 0;
  errno = 0;

  if (-1 < (res = send(fd, buf, length, 0))) {
    *count_out = (unsigned int) res;
    return 1;
  }
  else if (EWOULDBLOCK == errno || EAGAIN == errno || ENOMEM == errno || ENOBUFS == errno) {
     send_err_count++;
     return 0;
  } else if (EINTR == errno) {
     send_eintr_count++;
     return 0;
  } else if (EPIPE == errno) {
  	 send_epipe_count++;
  	 return 0;
  }
  return -1;
}


void
myclose(int fd)
{
   int tryed = 0;
   
   errno = 0;
   while (close(fd) == -1) {
      
      Debug((DBLMAX, "close(fd) failed (%d)...", errno));
      
      if (EBADF == errno)
         break;
      
      /* usleep(50000); --not nice */
      
      if(++tryed >= 10) {
         log_error("mp_utils:myclose error (errno == %d)", errno);
         Debug((DBLMAX, "close(fd) failed tryed (%d) again...", tryed));
         break;
      }
   }
}

/* mp_utils.c */
