/* Pxys - An IRC ProxyScan, ms_auth.c
 * 
 * Copyright (C) 2001,2002 Stephane Thiell
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * IRC (server) authentication.
 *
 * $Id: ms_auth.c,v 1.10 2002/12/16 00:33:30 mbuna Exp $
 */


#define NICKCHANGE_OFFSET_TIME   0 /* 86400 almost assure no nick collisions
                                    * but fuck up the lag delay in the /map
                                    */
#define BIRTHTIME                1000000000L

/* Channel console TimeStamp.
 */
#define CONSOLE_TS  1004078035

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>

#include "numnicks.h"
#include "typedefs.h"
#include "conf.h"

#include "ms_auth.h"
#include "ms_msgout.h"
#include "ms_globals.h"
#include "m_array.h"     /* array_findclient_from_nick */


time_t burst_starttime;
extern char MyBin[32];
#ifdef N2K_SUPPORT
const char MyClientXXX[] = "AAA";
#else
const char MyClientXXX[] = "AA";
#endif

void
irc_auth(struct Uplink_s *uplink)
{
   char charXXX[4];
   char MyClientIP[20];
#ifdef RAND_NICK
   char new_nick[NICKLEN + 1];
#endif
   time_t now;
   
   MyUpNum = 0;
   
   time(&now);
   
#ifdef N2K_SUPPORT
   inttobase64(MyYY, Me->numeric, 2);
   inttobase64(charXXX, Me->capacity, 3);
#else
   inttobase64(MyYY, Me->numeric, 1);
   inttobase64(charXXX, Me->capacity, 2);
#endif
   
   sendto_uplink("%s %s", MSG_PASS, uplink->password);
   
#ifdef N2K_SUPPORT
   sendto_uplink("%s %s %d %ld %ld J10 %s%s +%s :%s", MSG_SERVER,
         Me->name, 1, BIRTHTIME, now, MyYY, charXXX, "s", Me->serverinfo);
#else
   sendto_uplink("%s %s %d %ld %ld J10 %s%s :%s", MSG_SERVER,
         Me->name, 1, BIRTHTIME, now, MyYY, charXXX, Me->serverinfo);
#endif
   
   /* Burst reply starts here */
   
   burst_starttime = now;
   
   /* MyClient? */
   if (NULL != MyClient->hostname) {

#ifdef RAND_NICK
      randnick_gen(new_nick);
#endif
      
      inttobase64(MyClientIP, MyClient->ip.s_addr, 6);
#ifdef RAND_NICK      
      strncpy(MyClient->nick, new_nick, NICKLEN+1);
#else
      strncpy(MyClient->nick, MyClient->nickname, NICKLEN+1);
#endif

#ifdef N2K_SUPPORT
      sendto_uplink("%s %s %s %d %ld %s %s %s %s %s%s :%s", MyYY,
            TOK_NICK, MyClient->nick, 1, BIRTHTIME, MyClient->username, MyClient->hostname,
            MyClient->mode, MyClientIP, MyYY, MyClientXXX, MyClient->realname);
#else
      sendto_uplink("%s %s %s %d %ld %s %s %s %s %s%s :%s", MyYY,
            MSG_NICK, MyClient->nick, 1, BIRTHTIME, MyClient->username, MyClient->hostname,
            MyClient->mode, MyClientIP, MyYY, MyClientXXX, MyClient->realname);
#endif

      if (MyClient->join)
#ifdef N2K_SUPPORT
         sendto_uplink("%s %s %s %ld %s %s%s:o", MyYY, TOK_BURST, 
            MyConsole->channel, CONSOLE_TS, MyConsole->mode, MyYY, MyClientXXX);
#else
         sendto_uplink("%s %s %s %ld %s %s%s:o", MyYY, MSG_BURST, 
            MyConsole->channel, CONSOLE_TS, MyConsole->mode, MyYY, MyClientXXX);
#endif
   }
   
#ifdef N2K_SUPPORT
   sendto_uplink("%s %s", MyYY, TOK_END_OF_BURST);
#else
   sendto_uplink("%s %s", MyYY, MSG_END_OF_BURST);
#endif
}

void
do_nickchange()
{
   struct Client *cptr;
   char new_nick[NICKLEN + 1];
   time_t now;
   
   /* We need to find an available nickname */
   do {
      randnick_gen(new_nick);
      cptr = array_findclient_from_nick(new_nick);
   } while(cptr != NULL) ;
   
   time(&now);
   
#ifdef N2K_SUPPORT
   /* AyAAE N mbuna 984944794 */
   sendto_uplink("%s%s %s %s %ld", MyYY, MyClientXXX, TOK_NICK,
         new_nick, now - NICKCHANGE_OFFSET_TIME);
#else
   sendto_uplink("%s%s %s %s %ld", MyYY, MyClientXXX, MSG_NICK,
         new_nick, now - NICKCHANGE_OFFSET_TIME);
#endif
   
   strncpy(MyClient->nick, new_nick, NICKLEN+1);
}


void
randnick_gen(char *nickbuf)
{
   int length;
   char numbuf[ 16 ];
   
   length = strlen(MyClient->nickname);
   
   strncpy(nickbuf, MyClient->nickname, 5);
   nickbuf[ 5 ] = 0;
   
   if(length > 5)
      length = 5;
   
   memset(numbuf, 0, sizeof(numbuf));
   snprintf(numbuf, sizeof(numbuf), "%lu", (unsigned long) rand()*rand());
   
   numbuf[9  - length] = 0;
   
   strcat(nickbuf, numbuf);
}



/* ms_auth.c */
